"""nXML Document objects."""

from Globals import HTMLFile
from OFS.DTMLDocument import DTMLDocument
from OFS.PropertyManager import PropertyManager
from urllib import quote
import Globals
import pythoncom
from win32com.client import Dispatch
import time
import string
done='done'


class nXMLDocument(PropertyManager, DTMLDocument):
	"""nXML Document objects are DocumentTemplate.HTML objects that act
	   as methods whose 'self' is the nXML Document itself."""

	meta_type='nXML Document'
	icon	 ='p_/dtmldoc'

	manage_options=( DTMLDocument.manage_options)

	__ac_permissions__=(
		('Change nXML Documents',
		 ('manage_editForm', 'manage', 'manage_main', 'manage_uploadForm',
		  'manage_edit', 'manage_upload', 'PUT',
		  'manage_historyCopy',
		  'manage_beforeHistoryCopy', 'manage_afterHistoryCopy',
		  )
		 ),
		)

	manage_editForm=HTMLFile('nXMLdocumentEdit', globals())
	manage_uploadForm=HTMLFile('nXMLdocumentUpload', globals())
	manage=manage_main=manage_editDocument=manage_editForm
	manage_proxyForm=HTMLFile('nXMLdocumentProxy', globals())

	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
		"""Render the document given a client object, REQUEST mapping,
		Response, and key word arguments."""
		xml=DTMLDocument.__call__(self,client=client,REQUEST=REQUEST,RESPONSE=RESPONSE,kw=kw)
		stylesheetName=getattr(REQUEST,'stylesheet',None)
		if stylesheetName is None:
			ses=getattr(self,'SessionDataManager',None)
			if ses:	
				ses=ses.getSessionData()
				stylesheetName=ses.get('stylesheet')
		if stylesheetName is None:
			stylesheetName=getattr(self,'stylesheet',None)
		if stylesheetName is None:
			return 'Stylesheet not found!'
		xslObj=getattr(self,stylesheetName,None)
		if not xslObj:
			return 'Stylesheet not found!'
		
		xsl=xslObj.__call__(client=self,REQUEST=REQUEST,RESPONSE=RESPONSE,kw=kw)
		if getattr(REQUEST,'xmlonly',None):
			return xml, xsl
		return self.ProcessStrings(xsl,xml)

	def reportParseError(self,error):
		s = ""
		for i in range(0,error.linepos):
			s=s+" "
		r = "<font face=Verdana size=2><font size=4>XML Error loading '%s'</font><P><B>%s</B></P></font>" % (error.url,error.reason)
		if (error.line > 0):
			r=r+"<font size=3><XMP> at line %s, character %s\n%s\n%s^</XMP></font>" % (error.line,error.linepos,error.srcText,s)
		return r

	def reportRuntimeError(self,exception):
		return "<font face=Verdana size=2><font size=4>XSL Runtime Error</font><P><B>%s</B></P></font>" % exception

	def ProcessStrings(self,xsl='',xml=''):
		pythoncom.CoInitialize()
		source = Dispatch("Microsoft.XMLDOM")
		source.async = 1==0
		source.loadXML(xml)
		style = Dispatch("Microsoft.XMLDOM")
		style.async = 1==0
		style.loadXML(xsl)
		if source.parseError.errorCode != 0:
			result = self.reportParseError(source.parseError)
		if style.parseError.errorCode != 0:
			result = self.reportParseError(style.parseError)
		try:
			result = source.transformNode(style)
		except Exception,e:
			result = self.reportRuntimeError(e)
		return result

Globals.default__class_init__(nXMLDocument)

default_dd_html="""<document>
<heading2><dtml-var title_or_id></heading2>
<para>This is the <dtml-var id> Document.</para>
</document>"""

addForm=HTMLFile('nXMLdocumentAdd', globals())

def addnXMLDocument(self, id, title='', file='', cacheon=0, cachetime=0, REQUEST=None, submit=None):
	"""Add a DTML Document object with the contents of file. If 'file' is empty, default document text is used."""
	if type(file) is not type(''): file=file.read()
	if not file: file=default_dd_html
	id=str(id)
	title=str(title)
	try:
		cacheon=int(cacheon)
	except:
		cacheon=0
	try:
		cachetime=int(cachetime)
	except:
		cachetime=0
	ob=nXMLDocument(file, __name__=id)
	ob.title=title
	ob.cacheon=cacheon
	ob.cachetime=cachetime
	id=self._setObject(id, ob)
	if REQUEST is not None:
		try: u=self.DestinationURL()
		except: u=REQUEST['URL1']
		if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
		REQUEST.RESPONSE.redirect(u+'/manage_main')
	return ''
